@@ -30,19 +30,24 @@ struct b2b_api media_b2b;
 struct rtp_relay_binds media_rtp;
 
 static str b2b_media_exchange_cap = str_init("media_exchange");
+str media_default_instance = str_init(MEDIA_DEFAULT_INSTANCE);
 
+static int mod_preinit(void);
 static int mod_init(void);
 static int media_fork_to_uri(struct sip_msg *msg, str *uri,
-		int leg, str *headers, int *medianum);
+		int leg, str *headers, int *medianum, str *instance);
 static int media_fork_from_call(struct sip_msg *msg, str *callid,
-		int leg, int *medianum);
-static int media_fork_pause(struct sip_msg *msg, int leg, int *medianum);
-static int media_fork_resume(struct sip_msg *msg, int leg, int *medianum);
+		int leg, int *medianum, str *instance);
+static int media_fork_pause(struct sip_msg *msg,
+		int leg, int *medianum, str *instance);
+static int media_fork_resume(struct sip_msg *msg,
+		int leg, int *medianum, str *instance);
 static int media_exchange_from_uri(struct sip_msg *msg, str *uri,
 		int leg, str *body, str *headers, int *nohold);
 static int media_exchange_to_call(struct sip_msg *msg, str *callid,
 		int leg, int *nohold);
-static int media_terminate(struct sip_msg *msg, int leg, int *nohold);
+static int media_terminate(struct sip_msg *msg,
+		int leg, int *nohold, str *instance);
 static int media_indialog(struct sip_msg *msg);
 static int fixup_media_leg(void **param);
 static int fixup_media_leg_both(void **param);
@@ -96,24 +101,29 @@ static const cmd_export_t cmds[] = {
 		{CMD_PARAM_STR|CMD_PARAM_OPT,fixup_media_leg_both,0}, /* leg */
 		{CMD_PARAM_STR|CMD_PARAM_OPT,0,0}, /* headers */
 		{CMD_PARAM_INT|CMD_PARAM_OPT,0,0}, /* medianum */
+		{CMD_PARAM_STR|CMD_PARAM_OPT,0,0}, /* instance */
 		{0,0,0}}, ALL_ROUTES},
 	{"media_fork_from_call",(cmd_function)media_fork_from_call, {
 		{CMD_PARAM_STR,0,0}, /* callid */
 		{CMD_PARAM_STR|CMD_PARAM_OPT,fixup_media_leg_both,0}, /* leg */
 		{CMD_PARAM_INT|CMD_PARAM_OPT,0,0}, /* medianum */
+		{CMD_PARAM_STR|CMD_PARAM_OPT,0,0}, /* instance */
 		{0,0,0}},
 		REQUEST_ROUTE|BRANCH_ROUTE|ONREPLY_ROUTE|FAILURE_ROUTE},
 	{"media_fork_pause",(cmd_function)media_fork_pause, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT,fixup_media_leg_both,0}, /* leg */
 		{CMD_PARAM_INT|CMD_PARAM_OPT,0,0}, /* medianum */
+		{CMD_PARAM_STR|CMD_PARAM_OPT,0,0}, /* instance */
 		{0,0,0}}, ALL_ROUTES},
 	{"media_fork_resume",(cmd_function)media_fork_resume, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT,fixup_media_leg_both,0}, /* leg */
 		{CMD_PARAM_INT|CMD_PARAM_OPT,0,0}, /* medianum */
+		{CMD_PARAM_STR|CMD_PARAM_OPT,0,0}, /* instance */
 		{0,0,0}}, ALL_ROUTES},
 	{"media_terminate",(cmd_function)media_terminate, {
 		{CMD_PARAM_STR|CMD_PARAM_OPT,fixup_media_leg,0}, /* leg */
 		{CMD_PARAM_INT|CMD_PARAM_OPT,0,0}, /* nohold */
+		{CMD_PARAM_STR|CMD_PARAM_OPT,0,0}, /* instance */
 		{0,0,0}}, ALL_ROUTES},
 	{"media_handle_indialog",(cmd_function)media_indialog, {
 		{0,0,0}},
@@ -132,6 +142,9 @@ static const mi_export_t mi_cmds[] = {
 		{mi_media_fork_from_call_to_uri, {"callid", "uri", "leg", 0}},
 		{mi_media_fork_from_call_to_uri, {"callid", "uri", "headers", 0}},
 		{mi_media_fork_from_call_to_uri, {"callid", "uri", "leg", "headers", 0}},
+		{mi_media_fork_from_call_to_uri, {"callid", "uri", "leg", "instance", 0}},
+		{mi_media_fork_from_call_to_uri, {"callid", "uri", "headers", "instance", 0}},
+		{mi_media_fork_from_call_to_uri, {"callid", "uri", "leg", "headers", "instance", 0}},
 		{EMPTY_MI_RECIPE}}
 	},
 	{ "media_exchange_from_call_to_uri", 0, 0, 0, {
@@ -153,6 +166,9 @@ static const mi_export_t mi_cmds[] = {
 		{mi_media_terminate, {"callid", "leg", 0}},
 		{mi_media_terminate, {"callid", "nohold", 0}},
 		{mi_media_terminate, {"callid", "leg", "nohold", 0}},
+		{mi_media_terminate, {"callid", "leg", "instance", 0}},
+		{mi_media_terminate, {"callid", "nohold", "instance", 0}},
+		{mi_media_terminate, {"callid", "leg", "nohold", "instance", 0}},
 		{EMPTY_MI_RECIPE}}
 	},
 	{EMPTY_MI_EXPORT}
@@ -174,7 +190,7 @@ struct module_exports exports = {
 	0,								/* exported pseudo-variables */
 	0,								/* extra processes */
 	0,								/* extra transformations */
-	0,								/* module pre-initialization function */
+	mod_preinit,							/* module pre-initialization function */
 	mod_init,						/* module initialization function */
 	NULL,							/* response handling function */
 	NULL,							/* destroy function */
@@ -182,13 +198,8 @@ struct module_exports exports = {
 	0								/* reload confirm function */
 };
 
-/**
- * init module function
- */
-static int mod_init(void)
+static int mod_preinit(void)
 {
-	LM_DBG("initializing media_exchange module ...\n");
-
 	if (load_dlg_api(&media_dlg) != 0) {
 		LM_ERR("dialog module not loaded! Cannot use media bridging module\n");
 		return -1;
@@ -205,6 +216,24 @@ static int mod_init(void)
 		return -1;
 	}
 
+	if (load_rtp_relay(&media_rtp) != 0)
+		LM_DBG("rtp_relay module not loaded! Cannot use streaming module\n");
+
+	if (init_media_sessions() < 0) {
+		LM_ERR("could not initialize media sessions!\n");
+		return -1;
+	}
+
+	return 0;
+}
+
+/**
+ * init module function
+ */
+static int mod_init(void)
+{
+	LM_DBG("initializing media_exchange module ...\n");
+
 	if (media_b2b.register_cb(media_exchange_event_received,
 			B2BCB_RECV_EVENT, &b2b_media_exchange_cap) < 0) {
 		LM_ERR("could not register loaded callback!\n");
@@ -217,15 +246,6 @@ static int mod_init(void)
 		return -1;
 	}
 
-
-	if (load_rtp_relay(&media_rtp) != 0)
-		LM_DBG("rtp_relay module not loaded! Cannot use streaming module\n");
-
-	if (init_media_sessions() < 0) {
-		LM_ERR("could not initialize media sessions!\n");
-		return -1;
-	}
-
 	return 0;
 }
 
@@ -271,7 +291,7 @@ static int fixup_media_leg_both(void **param)
 	return 0;
 }
 
-static inline client_info_t *media_get_client_info(struct socket_info *si,
+static inline client_info_t *media_get_client_info(const struct socket_info *si,
 		str *uri, str *hdrs, str *body)
 {
 	static client_info_t ci;
@@ -291,7 +311,7 @@ static inline client_info_t *media_get_client_info(struct socket_info *si,
 	return &ci;
 }
 
-static int handle_media_fork_to_uri(struct media_session_leg *msl, struct socket_info *si,
+static int handle_media_fork_to_uri(struct media_session_leg *msl, const struct socket_info *si,
 		str *uri, str *headers, int medianum)
 {
 	static client_info_t *ci;
@@ -299,10 +319,11 @@ static int handle_media_fork_to_uri(struct media_session_leg *msl, struct socket
 	str *b2b_key, body;
 
 	MEDIA_LEG_LOCK(msl);
+
 	if (msl->params) {
-		LM_WARN("already an ongoing forking for this leg!\n");
+		LM_DBG("already an ongoing forking for this leg, probably retransmission!\n");
 		MEDIA_LEG_UNLOCK(msl);
-		goto destroy;
+		return 0;
 	}
 	mf = media_get_fork_sdp(msl, medianum, &body);
 	if (!mf) {
@@ -342,7 +363,7 @@ destroy:
 
 struct media_fork_params {
 	struct media_session_leg *msl;
-	struct socket_info *si;
+	const struct socket_info *si;
 	str uri;
 	str headers;
 	int medianum;
@@ -368,10 +389,10 @@ static void media_fork_start(struct cell *t, int type, struct tmcb_params *ps)
 }
 
 static int media_fork_to_uri(struct sip_msg *msg,
-		str *uri, int leg, str *headers, int *medianum)
+		str *uri, int leg, str *headers, int *medianum, str *instance)
 {
 	struct dlg_cell *dlg;
-	struct socket_info *si;
+	const struct socket_info *si;
 	struct media_session_leg *msl;
 	struct media_fork_params *mp;
 
@@ -401,7 +422,7 @@ static int media_fork_to_uri(struct sip_msg *msg,
 			leg = MEDIA_LEG_CALLEE;
 	}
 
-	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_FORK, leg, 0);
+	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_FORK, leg, 0, MEDIA_INSTANCE(instance));
 	if (!msl) {
 		LM_ERR("cannot create new exchange leg!\n");
 		return -2;
@@ -444,7 +465,7 @@ static int media_fork_to_uri(struct sip_msg *msg,
 			goto destroy;
 		}
 	} else if (dlg->state < DLG_STATE_DELETED) {
-		if (handle_media_fork_to_uri(msl, si, uri, headers, (medianum?*medianum:-1)) < 0) {
+		if (handle_media_fork_to_uri(msl, si, uri, headers, (medianum?*medianum:-1)) <= 0) {
 			LM_ERR("could not start media forking!\n");
 			goto destroy;
 		}
@@ -460,7 +481,8 @@ destroy:
 	return -3;
 }
 
-static int media_fork_from_call(struct sip_msg *msg, str *callid, int leg, int *medianum)
+static int media_fork_from_call(struct sip_msg *msg,
+		str *callid, int leg, int *medianum, str *instance)
 {
 	str contact;
 	str *b2b_key;
@@ -498,7 +520,7 @@ static int media_fork_from_call(struct sip_msg *msg, str *callid, int leg, int *
 		return -2;
 	}
 
-	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_FORK, leg, 0);
+	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_FORK, leg, 0, MEDIA_INSTANCE(instance));
 	if (!msl) {
 		LM_ERR("cannot create new fetch leg!\n");
 		goto unref;
@@ -585,7 +607,7 @@ static void media_session_tm_free(struct media_session_tm_param *p)
 	shm_free(p);
 }
 
-static int handle_media_exchange_from_uri(struct socket_info *si, struct dlg_cell *dlg,
+static int handle_media_exchange_from_uri(const struct socket_info *si, struct dlg_cell *dlg,
 		str *uri, int leg, str *body, str *headers, int nohold,
 		rtp_ctx ctx, struct media_session_tm_param *p)
 {
@@ -593,7 +615,7 @@ static int handle_media_exchange_from_uri(struct socket_info *si, struct dlg_cel
 	static client_info_t *ci;
 	str *b2b_key;
 
-	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_EXCHANGE, leg, nohold);
+	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_EXCHANGE, leg, nohold, NULL);
 	if (!msl) {
 		LM_ERR("cannot create new exchange leg!\n");
 		return -2;
@@ -641,7 +663,7 @@ static int media_exchange_from_uri(struct sip_msg *msg, str *uri, int leg,
 	struct cell *t = NULL;
 	struct dlg_cell *dlg;
 	int req_leg;
-	struct socket_info *si;
+	const struct socket_info *si;
 	struct media_session_tm_param *p = NULL;
 	rtp_ctx ctx = NULL;
 	int release = 0;
@@ -675,9 +697,10 @@ static int media_exchange_from_uri(struct sip_msg *msg, str *uri, int leg,
 	if (!body) {
 		if (media_rtp.get_ctx_dlg) {
 			ctx = media_rtp.get_ctx_dlg(dlg);
-			body = media_exchange_get_offer_sdp(ctx, dlg, leg, &release);
+			body = media_exchange_get_offer_sdp(ctx, dlg,
+					DLG_MEDIA_SESSION_LEG(dlg, leg), &release);
 		} else {
-			sbody = dlg_get_out_sdp(dlg, req_leg);
+			sbody = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_OTHER_LEG(dlg, leg));
 			body = &sbody;
 		}
 	}
@@ -710,14 +733,34 @@ static int media_exchange_from_uri(struct sip_msg *msg, str *uri, int leg,
 	return 1;
 }
 
+static void media_session_exchange_server_release(void *param)
+{
+	struct media_session_leg **mslp = (struct media_session_leg **)param;
+	if (!mslp) {
+		LM_BUG("media_session_leg should be here!\n");
+		return;
+	}
+	if (*mslp) {
+		MSL_UNREF((*mslp));
+		*mslp = NULL;
+	}
+	shm_free(mslp);
+}
+
+/* used just to indicate whether the session has been handled or not */
 static int media_session_exchange_server_reply(struct sip_msg *msg, int status, void *param)
 {
-	struct media_session_leg *msl;
+	struct media_session_leg *msl, **mslp;
 	str reason, body, *pbody;
+	int ret = -1;
 
 	if (status < 200) /* don't mind about provisional */
 		return 0;
-	msl = (struct media_session_leg *)param;
+	mslp = (struct media_session_leg **)param;
+
+	if (*mslp == NULL)
+		return 0;
+	msl = *mslp;
 
 	/* final reply here - unref the session */
 	if (msg == FAKED_REPLY || status >= 300)
@@ -740,7 +783,7 @@ static int media_session_exchange_server_reply(struct sip_msg *msg, int status,
 		/* we need to put the other party on hold */
 		pbody = media_session_get_hold_sdp(msl);
 		if (!pbody)
-			goto error;
+			return -1; /* we don't unref - a reply might get through */
 		/* XXX: should we care whether the other party is properly on hold? */
 		if (media_session_reinvite(msl,
 				MEDIA_SESSION_DLG_OTHER_LEG(msl), pbody) < 0)
@@ -749,23 +792,19 @@ static int media_session_exchange_server_reply(struct sip_msg *msg, int status,
 	}
 
 	/* finished processing this reply */
-	MSL_UNREF(msl);
-	return 0;
+	ret = 0;
+	goto end;
 
 terminate:
 	/* the client declined the invite - propagate the code */
 	reason.s = error_text(status);
 	reason.len = strlen(reason.s);
 	media_session_rpl(msl, METHOD_INVITE, status, &reason, NULL);
-
 	MSL_UNREF(msl);
-	/* no need of this session leg - remote it */
-	media_session_leg_free(msl);
-	return -1;
-
-error:
+end:
+	*mslp = NULL;
 	MSL_UNREF(msl);
-	return -1;
+	return ret;
 }
 
 static int media_exchange_to_call(struct sip_msg *msg, str *callid, int leg, int *nohold)
@@ -774,7 +813,7 @@ static int media_exchange_to_call(struct sip_msg *msg, str *callid, int leg, int
 	str contact;
 	str *b2b_key;
 	struct dlg_cell *dlg;
-	struct media_session_leg *msl;
+	struct media_session_leg *msl, **mslp;
 	static str inv = str_init("INVITE");
 
 	if (leg == MEDIA_LEG_UNSPEC) {
@@ -804,7 +843,7 @@ static int media_exchange_to_call(struct sip_msg *msg, str *callid, int leg, int
 	}
 
 	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_EXCHANGE, leg,
-			((nohold && *nohold)?1:0));
+			((nohold && *nohold)?1:0), NULL);
 	if (!msl) {
 		LM_ERR("cannot create new exchange leg!\n");
 		goto unref;
@@ -823,10 +862,19 @@ static int media_exchange_to_call(struct sip_msg *msg, str *callid, int leg, int
 		goto destroy;
 	}
 	msl->b2b_entity = B2B_SERVER;
+
+	mslp = shm_malloc(sizeof *mslp);
+	if (!mslp) {
+		LM_ERR("oom for new mslp\n");
+		goto destroy;
+	}
+	*mslp = msl;
+
 	/* all good - send the invite to the client */
 	MSL_REF(msl);
-	if (media_dlg.send_indialog_request(dlg, &inv, MEDIA_SESSION_DLG_LEG(msl),
-			&body, &msg->content_type->body, NULL, media_session_exchange_server_reply, msl) < 0) {
+	if (media_dlg.send_indialog_request(dlg, &inv, MEDIA_SESSION_DLG_LEG(msl), &body,
+			&msg->content_type->body, NULL, media_session_exchange_server_reply, mslp,
+			media_session_exchange_server_release) < 0) {
 		LM_ERR("could not send indialog request for callid %.*s\n", callid->len, callid->s);
 		MSL_UNREF(msl);
 		goto destroy;
@@ -841,7 +889,7 @@ unref:
 	return -2;
 }
 
-static int media_terminate(struct sip_msg *msg, int leg, int *nohold)
+static int media_terminate(struct sip_msg *msg, int leg, int *nohold, str *instance)
 {
 	struct dlg_cell *dlg;
 	struct media_session *ms;
@@ -874,7 +922,7 @@ static int media_terminate(struct sip_msg *msg, int leg, int *nohold)
 			proxied = 1;
 		}
 	}
-	if (media_session_end(ms, leg, ((nohold && *nohold)?1:0), proxied) < 0) {
+	if (media_session_end(ms, leg, ((nohold && *nohold)?1:0), proxied, instance) < 0) {
 		LM_ERR("could not terminate media session!\n");
 		return -2;
 	}
@@ -933,7 +981,7 @@ static int handle_media_indialog_fork(struct sip_msg *msg,
 {
 	MEDIA_LEG_LOCK(msl);
 	if (msl->state != MEDIA_SESSION_STATE_RUNNING) {
-		LM_DBG("this media leg is already involved in a different negociation! "
+		LM_DBG("this media leg is already involved in a different negotiation! "
 				"state=%d\n", msl->state);
 		MEDIA_LEG_UNLOCK(msl);
 		return -2; /* drop this request */
@@ -955,35 +1003,45 @@ static int handle_media_indialog_fork(struct sip_msg *msg,
 static int handle_media_indialog_refresh(struct sip_msg *msg,
 		struct media_session *ms, str *body)
 {
-	int ret = -1;
+	int ret = -1, ret_tmp;
 	str sbody;
 	int req_leg;
 	struct cell *t;
 	struct media_session_tm_param *p;
-	struct media_session_leg *leg, *oleg;
+	struct media_session_leg *leg, *tleg, *oleg;
+	tleg = oleg = NULL;
+
+	t = media_tm.t_gett();
+	if (t == T_UNDEFINED)
+		t = NULL;
 
 	if (media_dlg.get_direction() == DLG_DIR_DOWNSTREAM)
 		req_leg = DLG_CALLER_LEG;
 	else
 		req_leg = callee_idx(ms->dlg);
-	leg = media_session_get_leg(ms,
-			(req_leg==DLG_CALLER_LEG?MEDIA_LEG_CALLER:MEDIA_LEG_CALLEE));
-	oleg = media_session_get_leg(ms,
-			(req_leg==DLG_CALLER_LEG?MEDIA_LEG_CALLEE:MEDIA_LEG_CALLER));
-	if (!leg && !oleg) {
-		LM_DBG("no legs involved!\n");
-		return -1;
-	}
-	t = media_tm.t_gett();
-	if (t == T_UNDEFINED)
-		t = NULL;
 
-	if (!leg) {
+	/* we shall iterate on all legs and decide what to do depending on their type */
+	for (leg = ms->legs; leg; leg = leg->next) {
+		if (leg->type == MEDIA_SESSION_TYPE_FORK) {
+			ret_tmp = handle_media_indialog_fork(msg, leg);
+			if (ret_tmp == -2)
+				ret = -2;
+			else if (ret_tmp < ret)
+				ret = ret_tmp;
+		} else {
+			/* we've got an exchange session - check if tleg or oleg */
+			if ((req_leg == DLG_CALLER_LEG && leg->leg == MEDIA_LEG_CALLER) ||
+					(req_leg != DLG_CALLER_LEG && leg->leg == MEDIA_LEG_CALLEE))
+				tleg = leg;
+			else
+				oleg = leg;
+		}
+	}
+
+	if (!tleg) {
 		/* here, we have a sequential request, but this leg is not involved in
 		 * any media session */
-		if (oleg->type == MEDIA_SESSION_TYPE_FORK) {
-			return handle_media_indialog_fork(msg, oleg);
-		} else {
+		if (oleg) {
 			/* we should reply with whatever reply we last sent to it and
 			 * drop the request - if this is not correct, but unfortunately
 			 * we can't bother the other leg with a re-invite, as he's already
@@ -999,23 +1057,8 @@ static int handle_media_indialog_refresh(struct sip_msg *msg,
 			}
 			return -2;
 		}
-	}
-
-	/* here, we still have the initial leg */
-	if (leg->type == MEDIA_SESSION_TYPE_FORK) {
-		ret = handle_media_indialog_fork(msg, leg);
-
-		if (oleg && oleg->type != MEDIA_SESSION_TYPE_FORK) {
-			/* reply to current leg whatever was last sent */
-			sbody = dlg_get_out_sdp(ms->dlg, other_leg(ms->dlg, req_leg));
-			media_send_ok(t, ms->dlg, req_leg, body);
-			ret = -2;
-		}
 	} else {
-		if (oleg && oleg->type == MEDIA_SESSION_TYPE_FORK)
-			handle_media_indialog_fork(msg, oleg);
-
-		/* here leg is part of an exchange - proxy it */
+		/* here, we still have the initial leg - it is part of an exchange - proxy it */
 		if (media_session_req(leg, INVITE, body) < 0) {
 			media_send_fail(t, ms->dlg, req_leg);
 			ret = -3;
@@ -1104,7 +1147,8 @@ static int media_send_fail(struct cell *t, struct dlg_cell *dlg, int leg)
 	return ret;
 }
 
-static int media_fork_pause(struct sip_msg *msg, int leg, int *medianum)
+static int media_fork_pause(struct sip_msg *msg,
+		int leg, int *medianum, str *instance)
 {
 	struct dlg_cell *dlg;
 	struct media_session *ms;
@@ -1125,15 +1169,20 @@ static int media_fork_pause(struct sip_msg *msg, int leg, int *medianum)
 		return -1;
 	}
 	if (leg == MEDIA_LEG_UNSPEC) {
-		for (msl = ms->legs; msl; msl = msl->next)
-			ret += media_fork_pause_resume(msl, medianum?*medianum:-1, 0);
+		for (msl = ms->legs; msl; msl = msl->next) {
+			if (media_session_match_leg(msl, msl->leg, MEDIA_SESSION_TYPE_FORK, instance))
+				ret += media_fork_pause_resume(msl, medianum?*medianum:-1, 0);
+		}
 	} else {
-		msl = media_session_get_leg(ms, leg);
+		msl = media_session_get_leg(ms, leg, MEDIA_SESSION_TYPE_FORK, instance);
 		if (!msl) {
 			LM_WARN("media session leg %d does not exist!\n", leg);
 			return -1;
 		}
-		ret = media_fork_pause_resume(msl, medianum?*medianum:-1, 0);
+		do {
+			ret += media_fork_pause_resume(msl, medianum?*medianum:-1, 0);
+			msl = media_session_get_next_leg(msl, leg, MEDIA_SESSION_TYPE_FORK, instance);
+		} while (msl);
 	}
 
 	if (ret == 0) {
@@ -1143,7 +1192,8 @@ static int media_fork_pause(struct sip_msg *msg, int leg, int *medianum)
 	return ret;
 }
 
-static int media_fork_resume(struct sip_msg *msg, int leg, int *medianum)
+static int media_fork_resume(struct sip_msg *msg,
+		int leg, int *medianum, str *instance)
 {
 	struct dlg_cell *dlg;
 	struct media_session *ms;
@@ -1164,15 +1214,20 @@ static int media_fork_resume(struct sip_msg *msg, int leg, int *medianum)
 		return -1;
 	}
 	if (leg == MEDIA_LEG_UNSPEC) {
-		for (msl = ms->legs; msl; msl = msl->next)
-			ret += media_fork_pause_resume(msl, medianum?*medianum:-1, 1);
+		for (msl = ms->legs; msl; msl = msl->next) {
+			if (media_session_match_leg(msl, msl->leg, MEDIA_SESSION_TYPE_FORK, instance))
+				ret += media_fork_pause_resume(msl, medianum?*medianum:-1, 1);
+		}
 	} else {
-		msl = media_session_get_leg(ms, leg);
+		msl = media_session_get_leg(ms, leg, MEDIA_SESSION_TYPE_FORK, instance);
 		if (!msl) {
 			LM_WARN("media session leg %d does not exist!\n", leg);
 			return -1;
 		}
-		ret = media_fork_pause_resume(msl, medianum?*medianum:-1, 1);
+		do {
+			ret += media_fork_pause_resume(msl, medianum?*medianum:-1, 1);
+			msl = media_session_get_next_leg(msl, leg, MEDIA_SESSION_TYPE_FORK, instance);
+		} while (msl);
 	}
 
 	if (ret == 0) {
@@ -1240,7 +1295,7 @@ static void handle_media_session_negative(struct media_session_leg *msl)
 		body = &sbody;
 	if (media_dlg.send_indialog_request(msl->ms->dlg,
 			&inv, dlg_leg, body, &content_type_sdp, NULL,
-			media_session_exchange_negative_reply, msl) < 0) {
+			media_session_exchange_negative_reply, msl, NULL) < 0) {
 		LM_ERR("could not forward INVITE!\n");
 		media_send_fail(p->t, msl->ms->dlg, dlg_leg);
 		msl->params = NULL;
@@ -1256,11 +1311,11 @@ static int handle_media_session_reply_exchange(struct media_session_leg *msl,
 	str sbody;
 	struct dlg_cell *dlg;
 
-	if (msl->ms->rtp)
-		body = media_exchange_get_answer_sdp(msl->ms->rtp, body,
-				msl->leg, &release);
-
 	dlg = msl->ms->dlg;
+	if (msl->ms->rtp)
+		body = media_exchange_get_answer_sdp(msl->ms->rtp, dlg, body,
+				MEDIA_SESSION_DLG_LEG(msl), &release);
+
 	if (!p) {
 		/* here we were triggered outside of a request - simply reinvite the
 		 * other leg with the new body */
@@ -1434,6 +1489,11 @@ static int b2b_media_notify(struct sip_msg *msg, str *key, int type,
 				MEDIA_LEG_LOCK(msl);
 				initial_state = msl->state;
 				MEDIA_LEG_UNLOCK(msl);
+				if (msg == FAKED_REPLY) {
+					LM_ERR("could not stream media due to timeout (callid=%.*s)\n",
+							msl->ms->dlg->callid.len, msl->ms->dlg->callid.s);
+					goto terminate;
+				}
 				if (msg->REPLY_STATUS >= 300) {
 					LM_ERR("could not stream media due to negative reply %d (callid=%.*s)\n",
 							msg->REPLY_STATUS, msl->ms->dlg->callid.len, msl->ms->dlg->callid.s);
@@ -1503,10 +1563,10 @@ static mi_response_t *mi_media_fork_from_call_to_uri(const mi_params_t *params,
 {
 	int medianum;
 	int media_leg;
-	str shdrs, *hdrs;
-	str callid, leg, uri;
+	str shdrs, *hdrs, *inst;
+	str callid, leg, uri, instance;
 	struct dlg_cell *dlg;
-	struct socket_info *si;
+	const struct socket_info *si;
 	union sockaddr_union tmp;
 	struct media_session_leg *msl;
 	rtp_ctx ctx = NULL;
@@ -1524,8 +1584,13 @@ static mi_response_t *mi_media_fork_from_call_to_uri(const mi_params_t *params,
 		medianum = -1;
 	if (try_get_mi_string_param(params, "headers", &shdrs.s, &shdrs.len) < 0)
 		hdrs = NULL;
+	if (try_get_mi_string_param(params, "instance", &instance.s, &instance.len) < 0)
+		inst = &media_default_instance;
 	else
-		hdrs = &shdrs;
+		inst = &instance;
+
+	if (try_get_mi_int_param(params, "medianum", &medianum) < 0)
+		medianum = -1;
 
 	media_leg = fixup_get_media_leg_both(&leg);
 	if (media_leg < 0)
@@ -1547,14 +1612,14 @@ static mi_response_t *mi_media_fork_from_call_to_uri(const mi_params_t *params,
 			return init_mi_error(404, MI_SSTR("Media context not found"));
 	}
 
-	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_FORK, media_leg, 0);
+	msl = media_session_new_leg(dlg, MEDIA_SESSION_TYPE_FORK, media_leg, 0, inst);
 	if (!msl) {
 		LM_ERR("cannot create new exchange leg!\n");
 		return init_mi_error(500, MI_SSTR("Could not create media forking"));
 	}
 	msl->ms->rtp = ctx;
 
-	if (handle_media_fork_to_uri(msl, si, &uri, hdrs, medianum) < 0) {
+	if (handle_media_fork_to_uri(msl, si, &uri, hdrs, medianum) <= 0) {
 		media_dlg.dlg_unref(dlg, 1);
 		return init_mi_error(500, MI_SSTR("Could not start media forking"));
 	}
@@ -1572,7 +1637,7 @@ static mi_response_t *mi_media_exchange_from_call_to_uri(const mi_params_t *para
 	str callid, leg, uri;
 	str body, shdrs, *hdrs, *pbody;
 	struct dlg_cell *dlg;
-	struct socket_info *si;
+	const struct socket_info *si;
 	union sockaddr_union tmp;
 	rtp_ctx ctx = NULL;
 	int release = 0;
@@ -1616,9 +1681,10 @@ static mi_response_t *mi_media_exchange_from_call_to_uri(const mi_params_t *para
 	if (try_get_mi_string_param(params, "body", &body.s, &body.len) < 0) {
 		if (media_rtp.get_ctx_dlg) {
 			ctx = media_rtp.get_ctx_dlg(dlg);
-			pbody = media_exchange_get_offer_sdp(ctx, dlg, media_leg, &release);
+			pbody = media_exchange_get_offer_sdp(ctx, dlg,
+					DLG_MEDIA_SESSION_LEG(dlg, media_leg), &release);
 		} else {
-			body = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_LEG(dlg, media_leg));
+			body = dlg_get_out_sdp(dlg, DLG_MEDIA_SESSION_OTHER_LEG(dlg, media_leg));
 			pbody = &body;
 		}
 	} else {
@@ -1629,12 +1695,12 @@ static mi_response_t *mi_media_exchange_from_call_to_uri(const mi_params_t *para
 			hdrs, nohold, ctx, NULL) < 0) {
 		media_dlg.dlg_unref(dlg, 1);
 		if (release)
-			pkg_free(&body.s);
+			pkg_free(pbody->s);
 		return init_mi_error(500, MI_SSTR("Could not start media session"));
 	}
 
 	if (release)
-		pkg_free(&body.s);
+		pkg_free(pbody->s);
 	/* all good now, unref the dialog as it is reffed by the ms */
 	media_dlg.dlg_unref(dlg, 1);
 	return init_mi_result_ok();
@@ -1645,7 +1711,7 @@ static mi_response_t *mi_media_terminate(const mi_params_t *params,
 {
 	int nohold;
 	int media_leg;
-	str callid, leg;
+	str callid, leg, instance, *inst = NULL;
 	struct dlg_cell *dlg;
 	struct media_session *ms;
 
@@ -1674,6 +1740,8 @@ static mi_response_t *mi_media_terminate(const mi_params_t *params,
 		default:
 			return init_mi_param_error();
 	}
+	if (try_get_mi_string_param(params, "instance", &instance.s, &instance.len) >= 0)
+		inst = &instance;
 
 	/* params are now ok, let's lookup the media session */
 	dlg = media_dlg.get_dlg_by_callid(&callid, 1);
@@ -1687,7 +1755,7 @@ static mi_response_t *mi_media_terminate(const mi_params_t *params,
 	}
 
 	/* all good - implement the logic now */
-	if (media_session_end(ms, media_leg, nohold, 0) < 0) {
+	if (media_session_end(ms, media_leg, nohold, 0, inst) < 0) {
 		media_dlg.dlg_unref(dlg, 1);
 		return init_mi_error(500, MI_SSTR("Terminate failed"));
 	}
