@@ -32,7 +32,9 @@ int cluster_federation = 0;
 
 str clustering_events = {NULL,0};
 
-static unsigned char clustered_events[EVENT_LINE_SEIZE];
+int *cluster_active = NULL;
+
+static unsigned char clustered_events[EVENT_LAST];
 
 struct clusterer_binds c_api;
 
@@ -49,10 +51,36 @@ static void bin_packet_handler(bin_packet_t *packet);
 static void cluster_event_handler(enum clusterer_event ev, int node_id);
 
 
-int init_pres_clustering(void)
+void cluster_active_shtag_cb(str *tag, int state, int c_id, void *param)
+{
+	if (cluster_active) {
+		*cluster_active = (state==SHTAG_STATE_ACTIVE)? 1 : 0;
+		if (cluster_federation == FEDERATION_FULL_SHARING && *cluster_active &&
+		c_api.request_sync(&presence_capability, pres_cluster_id, 0) < 0)
+			LM_ERR("Sync request failed\n");
+	}
+}
+
+int init_pres_clustering(char *federation_mode_str,
+												char *cluster_active_shtag_str)
 {
 	csv_record *list, *it;
 	event_t e;
+	str sh_tag;
+	int shtag_state;
+
+	if (!federation_mode_str || !strcasecmp(federation_mode_str, "disabled")) {
+		cluster_federation = FEDERATION_DISABLED;
+		pres_cluster_id = 0;
+		return 0;
+	} else if (!strcasecmp(federation_mode_str, "on-demand-sharing")) {
+		cluster_federation = FEDERATION_ON_DEMAND;
+	} else if (!strcasecmp(federation_mode_str, "full-sharing")) {
+		cluster_federation = FEDERATION_FULL_SHARING;
+	} else {
+		LM_ERR("invalid cluster_federation_mode: '%s'\n", federation_mode_str);
+		return -1;
+	}
 
 	/* is clustering needed ? */
 	if (!is_presence_cluster_enabled())
@@ -64,6 +92,13 @@ int init_pres_clustering(void)
 		return -1;
 	}
 
+	cluster_active = (int*)shm_malloc( sizeof(int) );
+	if (cluster_active==NULL) {
+		LM_ERR("not enough shm mem for cluster_active variable\n");
+		return -1;
+	}
+	*cluster_active = 1; /* default is active */
+
 	/* register handler for receiving packets from the clusterer module */
 	if (c_api.register_capability( &presence_capability,
 		bin_packet_handler, cluster_event_handler, pres_cluster_id,
@@ -73,6 +108,25 @@ int init_pres_clustering(void)
 		return -1;
 	}
 
+	if (cluster_active_shtag_str) {
+		sh_tag.s = cluster_active_shtag_str;
+		sh_tag.len = strlen(cluster_active_shtag_str);
+		if ((shtag_state=c_api.shtag_get( &sh_tag, pres_cluster_id))<0) {
+			LM_ERR("failed to lookup the <%.*s> active sharing tag\n",
+				sh_tag.len, sh_tag.s);
+			return -1;
+		}
+		/* inherite the active status*/
+		*cluster_active = (shtag_state==SHTAG_STATE_ACTIVE)? 1 : 0;
+
+		if (c_api.shtag_register_callback( &sh_tag, pres_cluster_id,
+		NULL, cluster_active_shtag_cb) < 0) {
+			LM_ERR("failed to register callbacks for <%.*s> sharing tag\n",
+				sh_tag.len, sh_tag.s);
+			return -1;
+		}
+	}
+
 	if (clustering_events.s) {
 		/* parse the event list  */
 		clustering_events.len = strlen(clustering_events.s);
@@ -96,7 +150,7 @@ int init_pres_clustering(void)
 		memset( clustered_events, 1, sizeof(clustered_events) );
 	}
 
-	if (cluster_federation == FEDERATION_FULL_SHARING &&
+	if (cluster_federation == FEDERATION_FULL_SHARING && *cluster_active &&
 		c_api.request_sync(&presence_capability, pres_cluster_id, 0) < 0)
 		LM_ERR("Sync request failed\n");
 
@@ -182,11 +236,11 @@ static int bin_push_presentity(bin_packet_t *packet, presentity_t *pres)
 		goto error;
 	step++;
 
-	if (bin_push_int(packet, pres->expires) < 0)
+	if (bin_push_int(packet, (int)(unsigned long)pres->expires) < 0)
 		goto error;
 	step++;
 
-	if (bin_push_int(packet, pres->received_time) < 0)
+	if (bin_push_int(packet, (int)(unsigned long)pres->received_time) < 0)
 		goto error;
 	step++;
 
@@ -234,6 +288,12 @@ void replicate_publish_on_cluster(presentity_t *pres)
 {
 	bin_packet_t packet;
 
+	/* am I active from clustering perspective? */
+	if (*cluster_active==0) {
+		LM_ERR("trying to do query cluster, but in inactive mode (according to sharing tag) :-/\n");
+		return;
+	}
+
 	memset( &packet, 0, sizeof(bin_packet_t) );
 	if (bin_init(&packet, &presence_capability,
 		CL_PRESENCE_PUBLISH, BIN_VERSION, 0) < 0)
@@ -258,6 +318,12 @@ void query_cluster_for_presentity(str *pres_uri, event_t *evp)
 	cluster_query_entry_t* p;
 	int step=0;
 
+	/* am I active from clustering perspective? */
+	if (*cluster_active==0) {
+		LM_ERR("trying to do query cluster, but in inactive mode (according to sharing tag) :-/\n");
+		return;
+	}
+
 	/* check if the presentity is not in the pending list */
 	hash_code= core_hash( pres_uri, NULL, phtable_size);
 	lock_get( &pres_htable[hash_code].lock );
@@ -512,8 +578,8 @@ int pack_repl_presentity(bin_packet_t *packet, str *pres_uri, pres_ev_t *ev)
 	pres.new_etag.s = (char*)VAL_STRING(ROW_VALUES(RES_ROWS(res))+etag_col);
 	pres.new_etag.len = strlen(pres.new_etag.s);
 	pres.expires = VAL_INT(ROW_VALUES(RES_ROWS(res))+expires_col) -
-		(int)time(NULL);
-	pres.received_time = (int)time(NULL);
+		(int)(unsigned long)time(NULL);
+	pres.received_time = (int)(unsigned long)time(NULL);
 	if (!VAL_NULL(ROW_VALUES(RES_ROWS(res))+extra_hdrs_col)) {
 		s.s = (char*)VAL_STRING(ROW_VALUES(RES_ROWS(res))+extra_hdrs_col);
 		s.len = strlen(s.s);
@@ -607,6 +673,10 @@ static void bin_packet_handler(bin_packet_t *pkt)
 {
 	int rc;
 
+	/* am I active from clustering perspective? */
+	if (*cluster_active==0)
+		return;
+
 	switch (pkt->type) {
 		case CL_PRESENCE_PUBLISH:
 			ensure_bin_version(pkt, BIN_VERSION);
@@ -686,6 +756,10 @@ error:
 
 static void cluster_event_handler(enum clusterer_event ev, int node_id)
 {
+	/* am I active from clustering perspective? */
+	if (*cluster_active==0)
+		return;
+
 	if (ev == SYNC_REQ_RCV && receive_sync_request(node_id) < 0)
 		LM_ERR("Failed to send sync data to node: %d\n", node_id);
 }
