@@ -122,6 +122,8 @@ static int pv_get_rl_count(struct sip_msg *msg, pv_param_t *param,
 		pv_value_t *res);
 static int pv_parse_rl_count(pv_spec_p sp, const str *in);
 
+static int fixup_avp(void** param);
+
 static const cmd_export_t cmds[] = {
 	{"rl_check", (cmd_function)w_rl_check, {
 		{CMD_PARAM_STR,0,0},
@@ -137,6 +139,10 @@ static const cmd_export_t cmds[] = {
 		{CMD_PARAM_STR,0,0}, {0,0,0}},
 		REQUEST_ROUTE|FAILURE_ROUTE|ONREPLY_ROUTE|
 		BRANCH_ROUTE|ERROR_ROUTE|LOCAL_ROUTE|TIMER_ROUTE|EVENT_ROUTE},
+	{"rl_values", (cmd_function)w_rl_values, {
+		{CMD_PARAM_VAR,fixup_avp,0},
+		{CMD_PARAM_REGEX|CMD_PARAM_OPT,0, 0}, {0,0,0}},
+		ALL_ROUTES},
 	{0,0,{{0,0,0}},0}
 };
 
@@ -194,7 +200,7 @@ static const mi_export_t mi_cmds [] = {
 };
 
 static const pv_export_t mod_items[] = {
-	{ {"rl_count", sizeof("rl_count")-1}, 1010, pv_get_rl_count, 0,
+	{ str_const_init("rl_count"), 1010, pv_get_rl_count, 0,
 		 pv_parse_rl_count, 0, 0, 0 },
 	{ {0, 0}, 0, 0, 0, 0, 0, 0, 0 }
 };
@@ -390,7 +396,7 @@ static int mod_init(void)
 	if (db_url.s) {
 		db_url.len = strlen(db_url.s);
 		db_prefix.len = strlen(db_prefix.s);
-		LM_DBG("using CacheDB url: %s\n", db_url.s);
+		LM_DBG("using CacheDB url: %s\n", db_url_escape(&db_url));
 	}
 
 	RL_SHM_MALLOC(rl_network_count, sizeof(int));
@@ -1127,3 +1133,13 @@ static int pv_parse_rl_count(pv_spec_p sp, const str *in)
 	return 0;
 
 }
+
+static int fixup_avp(void** param)
+{
+	if (((pv_spec_t*)*param)->type != PVT_AVP) {
+		LM_ERR("invalid pvar type - only AVPs are allowed!\n");
+		return E_SCRIPT;
+	}
+
+	return 0;
+}
