@@ -181,164 +181,164 @@ static const mi_export_t mi_cmds[] = {
  */
 static const pv_export_t mod_items[] = {
 	/* TLS session parameters */
-	{{"tls_version", sizeof("tls_version")-1},
+	{str_const_init("tls_version"),
 		850, tlsops_version, 0,
 		0, 0, 0, 0 },
-	{{"tls_description", sizeof("tls_description")-1},
+	{str_const_init("tls_description"),
 		850, tlsops_desc, 0,
 		0, 0, 0, 0 },
-	{{"tls_cipher_info", sizeof("tls_cipher_info")-1},
+	{str_const_init("tls_cipher_info"),
 		850, tlsops_cipher, 0,
 		0, 0, 0, 0 },
-	{{"tls_cipher_bits", sizeof("tls_cipher_bits")-1},
+	{str_const_init("tls_cipher_bits"),
 		850,  tlsops_bits, 0,
 		0, 0, 0, 0 },
 	/* general certificate parameters for peer and local */
-	{{"tls_peer_version", sizeof("tls_peer_version")-1},
+	{str_const_init("tls_peer_version"),
 		850, tlsops_cert_version, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  },
-	{{"tls_my_version", sizeof("tls_my_version")-1},
+	{str_const_init("tls_my_version"),
 		850, tlsops_cert_version, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL },
-	{{"tls_peer_serial", sizeof("tls_peer_serial")-1},
+	{str_const_init("tls_peer_serial"),
 		850, tlsops_sn, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  },
-	{{"tls_my_serial", sizeof("tls_my_serial")-1},
+	{str_const_init("tls_my_serial"),
 		850, tlsops_sn,0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL },
 	/* certificate parameters for peer and local, for subject and issuer*/
-	{{"tls_peer_subject", sizeof("tls_peer_subject")-1},
+	{str_const_init("tls_peer_subject"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT },
-	{{"tls_peer_issuer", sizeof("tls_peer_issuer")-1},
+	{str_const_init("tls_peer_issuer"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  },
-	{{"tls_my_subject", sizeof("tls_my_subject")-1},
+	{str_const_init("tls_my_subject"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT },
-	{{"tls_my_issuer", sizeof("tls_my_issuer")-1},
+	{str_const_init("tls_my_issuer"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  },
-	{{"tls_peer_subject_cn", sizeof("tls_peer_subject_cn")-1},
+	{str_const_init("tls_peer_subject_cn"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT | VAR_COMP_CN },
-	{{"tls_peer_issuer_cn", sizeof("tls_peer_issuer_cn")-1},
+	{str_const_init("tls_peer_issuer_cn"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  | VAR_COMP_CN },
-	{{"tls_my_subject_cn", sizeof("tls_my_subject_cn")-1},
+	{str_const_init("tls_my_subject_cn"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_CN },
-	{{"tls_my_issuer_cn", sizeof("tls_my_issuer_cn")-1},
+	{str_const_init("tls_my_issuer_cn"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  | VAR_COMP_CN },
-	{{"tls_peer_subject_locality", sizeof("tls_peer_subject_locality")-1},
+	{str_const_init("tls_peer_subject_locality"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT | VAR_COMP_L },
-	{{"tls_peer_issuer_locality", sizeof("tls_peer_issuer_locality")-1},
+	{str_const_init("tls_peer_issuer_locality"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  | VAR_COMP_L },
-	{{"tls_my_subject_locality", sizeof("tls_my_subject_locality")-1},
+	{str_const_init("tls_my_subject_locality"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_L },
-	{{"tls_my_issuer_locality", sizeof("tls_my_issuer_locality")-1},
+	{str_const_init("tls_my_issuer_locality"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  | VAR_COMP_L },
-	{{"tls_peer_subject_country", sizeof("tls_peer_subject_country")-1},
+	{str_const_init("tls_peer_subject_country"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT | VAR_COMP_C },
-	{{"tls_peer_issuer_country", sizeof("tls_peer_issuer_country")-1},
+	{str_const_init("tls_peer_issuer_country"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  | VAR_COMP_C },
-	{{"tls_my_subject_country", sizeof("tls_my_subject_country")-1},
+	{str_const_init("tls_my_subject_country"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_C },
-	{{"tls_my_issuer_country", sizeof("tls_my_issuer_country")-1},
+	{str_const_init("tls_my_issuer_country"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  | VAR_COMP_C },
-	{{"tls_peer_subject_state", sizeof("tls_peer_subject_state")-1},
+	{str_const_init("tls_peer_subject_state"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT | VAR_COMP_ST },
-	{{"tls_peer_issuer_state", sizeof("tls_peer_issuer_state")-1},
+	{str_const_init("tls_peer_issuer_state"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  | VAR_COMP_ST },
-	{{"tls_my_subject_state", sizeof("tls_my_subject_state")-1},
+	{str_const_init("tls_my_subject_state"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_ST },
-	{{"tls_my_issuer_state", sizeof("tls_my_issuer_state")-1},
+	{str_const_init("tls_my_issuer_state"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  | VAR_COMP_ST },
-	{{"tls_peer_subject_organization", sizeof("tls_peer_subject_organization")-1},
+	{str_const_init("tls_peer_subject_organization"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT | VAR_COMP_O },
-	{{"tls_peer_issuer_organization", sizeof("tls_peer_issuer_organization")-1},
+	{str_const_init("tls_peer_issuer_organization"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  | VAR_COMP_O },
-	{{"tls_my_subject_organization", sizeof("tls_my_subject_organization")-1},
+	{str_const_init("tls_my_subject_organization"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_O },
-	{{"tls_my_issuer_organization", sizeof("tls_my_issuer_organization")-1},
+	{str_const_init("tls_my_issuer_organization"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  | VAR_COMP_O },
-	{{"tls_peer_subject_unit", sizeof("tls_peer_subject_unit")-1},
+	{str_const_init("tls_peer_subject_unit"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_SUBJECT | VAR_COMP_OU },
-	{{"tls_peer_issuer_unit", sizeof("tls_peer_issuer_unit")-1},
+	{str_const_init("tls_peer_issuer_unit"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_CERT_ISSUER  | VAR_COMP_OU },
-	{{"tls_my_subject_unit", sizeof("tls_my_subject_unit")-1},
+	{str_const_init("tls_my_subject_unit"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_OU },
-	{{"tls_my_subject_serial", sizeof("tls_my_subject_serial")-1},
+	{str_const_init("tls_my_subject_serial"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_SUBJECT | VAR_COMP_SUBJECT_SERIAL },
-	{{"tls_peer_subject_serial", sizeof("tls_peer_subject_serial")-1},
+	{str_const_init("tls_peer_subject_serial"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER | VAR_CERT_SUBJECT | VAR_COMP_SUBJECT_SERIAL },
-	{{"tls_my_issuer_unit", sizeof("tls_my_issuer_unit")-1},
+	{str_const_init("tls_my_issuer_unit"),
 		850, tlsops_comp, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_CERT_ISSUER  | VAR_COMP_OU },
 	/* subject alternative name parameters for peer and local */
-	{{"tls_peer_san_email", sizeof("tls_peer_san_email")-1},
+	{str_const_init("tls_peer_san_email"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_COMP_E },
-	{{"tls_my_san_email", sizeof("tls_my_san_email")-1},
+	{str_const_init("tls_my_san_email"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_COMP_E },
-	{{"tls_peer_san_hostname", sizeof("tls_peer_san_hostname")-1},
+	{str_const_init("tls_peer_san_hostname"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_COMP_HOST },
-	{{"tls_my_san_hostname", sizeof("tls_my_san_hostname")-1},
+	{str_const_init("tls_my_san_hostname"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_COMP_HOST },
-	{{"tls_peer_san_uri", sizeof("tls_peer_san_uri")-1},
+	{str_const_init("tls_peer_san_uri"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_COMP_URI },
-	{{"tls_my_san_uri", sizeof("tls_my_san_uri")-1},
+	{str_const_init("tls_my_san_uri"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_COMP_URI },
-	{{"tls_peer_san_ip", sizeof("tls_peer_san_ip")-1},
+	{str_const_init("tls_peer_san_ip"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_PEER  | VAR_COMP_IP },
-	{{"tls_my_san_ip", sizeof("tls_my_san_ip")-1},
+	{str_const_init("tls_my_san_ip"),
 		850, tlsops_alt, 0,
 		0, 0, pv_init_iname, VAR_CERT_LOCAL | VAR_COMP_IP },
 	/* peer certificate validation parameters */
-	{{"tls_peer_verified", sizeof("tls_peer_verified")-1},
+	{str_const_init("tls_peer_verified"),
 		850, tlsops_check_cert, 0,
 		0, 0, pv_init_iname, VAR_CERT_VERIFIED },
-	{{"tls_peer_revoked", sizeof("tls_peer_revoked")-1},
+	{str_const_init("tls_peer_revoked"),
 		850, tlsops_check_cert, 0,
 		0, 0, pv_init_iname, VAR_CERT_REVOKED },
-	{{"tls_peer_expired", sizeof("tls_peer_expired")-1},
+	{str_const_init("tls_peer_expired"),
 		850, tlsops_check_cert, 0,
 		0, 0, pv_init_iname, VAR_CERT_EXPIRED },
-	{{"tls_peer_selfsigned", sizeof("tls_peer_selfsigned")-1},
+	{str_const_init("tls_peer_selfsigned"),
 		850, tlsops_check_cert, 0,
 		0, 0, pv_init_iname, VAR_CERT_SELFSIGNED },
-	{{"tls_peer_notBefore", sizeof("tls_peer_notBefore")-1},
+	{str_const_init("tls_peer_notBefore"),
 		850, tlsops_validity, 0,
 		0, 0, pv_init_iname, VAR_CERT_NOTBEFORE },
-	{{"tls_peer_notAfter", sizeof("tls_peer_notAfter")-1},
+	{str_const_init("tls_peer_notAfter"),
 		850, tlsops_validity, 0,
 		0, 0, pv_init_iname, VAR_CERT_NOTAFTER },
 
