@@ -52,8 +52,8 @@
 static int mod_init(void);
 static int proto_tcp_init(struct proto_info *pi);
 static int proto_tcp_init_listener(struct socket_info *si);
-static int proto_tcp_send(struct socket_info* send_sock,
-		char* buf, unsigned int len, union sockaddr_union* to,
+static int proto_tcp_send(const struct socket_info* send_sock,
+		char* buf, unsigned int len, const union sockaddr_union* to,
 		unsigned int id);
 inline static int _tcp_write_on_socket(struct tcp_connection *c, int fd,
 		char *buf, int len);
@@ -203,12 +203,13 @@ static int proto_tcp_init(struct proto_info *pi)
 	pi->default_port		= tcp_port;
 
 	pi->tran.init_listener	= proto_tcp_init_listener;
+	pi->tran.bind_listener	= tcp_bind_listener;
 	pi->tran.send			= proto_tcp_send;
 	pi->tran.dst_attr		= tcp_conn_fcntl;
 
 	pi->net.flags			= PROTO_NET_USE_TCP;
-	pi->net.read			= (proto_net_read_f)tcp_read_req;
-	pi->net.write			= (proto_net_write_f)tcp_async_write;
+	pi->net.stream.read		= tcp_read_req;
+	pi->net.stream.write		= tcp_async_write;
 	pi->net.report			= tcp_report;
 
 	if (tcp_async && !tcp_has_async_write()) {
@@ -223,7 +224,7 @@ static int proto_tcp_init(struct proto_info *pi)
 		 * if the tcp_async is enbled */
 		if (tcp_async_max_postponed_chunks<=1)
 			tcp_async_max_postponed_chunks = 2;
-		pi->net.async_chunks= tcp_async_max_postponed_chunks;
+		pi->net.stream.async_chunks= tcp_async_max_postponed_chunks;
 	}
 
 	return 0;
@@ -360,9 +361,9 @@ inline static int _tcp_write_on_socket(struct tcp_connection *c, int fd,
 
 
 /*! \brief Finds a tcpconn & sends on it */
-static int proto_tcp_send(struct socket_info* send_sock,
+static int proto_tcp_send(const struct socket_info* send_sock,
 									char* buf, unsigned int len,
-									union sockaddr_union* to, unsigned int id)
+									const union sockaddr_union* to, unsigned int id)
 {
 	struct tcp_connection *c;
 	struct tcp_conn_profile prof;
@@ -442,8 +443,8 @@ static int proto_tcp_send(struct socket_info* send_sock,
 
 				/* mark the ID of the used connection (tracing purposes) */
 				last_outgoing_tcp_id = c->id;
-				send_sock->last_local_real_port = c->rcv.dst_port;
-				send_sock->last_remote_real_port = c->rcv.src_port;
+				send_sock->last_real_ports->local = c->rcv.dst_port;
+				send_sock->last_real_ports->remote = c->rcv.src_port;
 
 				/* connect is still in progress, break the sending
 				 * flow now (the actual write will be done when
@@ -532,8 +533,8 @@ static int proto_tcp_send(struct socket_info* send_sock,
 
 			/* mark the ID of the used connection (tracing purposes) */
 			last_outgoing_tcp_id = c->id;
-			send_sock->last_local_real_port = c->rcv.dst_port;
-			send_sock->last_remote_real_port = c->rcv.src_port;
+			send_sock->last_real_ports->local = c->rcv.dst_port;
+			send_sock->last_real_ports->remote = c->rcv.src_port;
 
 			/* we successfully added our write chunk - success */
 			sh_log(c->hist, TCP_SEND2MAIN, "send 3, (%d)", c->refcnt);
@@ -581,8 +582,8 @@ send_it:
 
 	/* mark the ID of the used connection (tracing purposes) */
 	last_outgoing_tcp_id = c->id;
-	send_sock->last_local_real_port = c->rcv.dst_port;
-	send_sock->last_remote_real_port = c->rcv.src_port;
+	send_sock->last_real_ports->local = c->rcv.dst_port;
+	send_sock->last_real_ports->remote = c->rcv.src_port;
 
 	sh_log(c->hist, TCP_SEND2MAIN, "send 6, (%d, async: %d)", c->refcnt, n < len);
 	tcp_conn_release(c, (n<len)?1:0/*pending data in async mode?*/ );
@@ -647,7 +648,7 @@ again:
  */
 static int tcp_read_req(struct tcp_connection* con, int* bytes_read)
 {
-	int bytes;
+	int bytes, rc;
 	int total_bytes;
 	struct tcp_req* req;
 
@@ -735,7 +736,7 @@ again:
 	int max_chunks = tcp_attr_isset(con, TCP_ATTR_MAX_MSG_CHUNKS) ?
 			con->profile.attrs[TCP_ATTR_MAX_MSG_CHUNKS] : tcp_max_msg_chunks;
 
-	switch (tcp_handle_req(req,con,max_chunks,parallel_handling)){
+	switch ((rc = tcp_handle_req(req,con,max_chunks,parallel_handling))){
 		case 1:
 			goto again;
 		case -1:
@@ -745,8 +746,9 @@ again:
 	LM_DBG("tcp_read_req end for conn %p, req is %p\n",con,con->con_req);
 done:
 	if (bytes_read) *bytes_read=total_bytes;
-	/* connection will be released */
-	return 0;
+
+	return rc == 2   ?  1  /* connection is already released! */
+	       /* 0,1? */:  0; /* connection will be released */
 error:
 	/* connection will be released as ERROR */
 	return -1;
