@@ -44,6 +44,7 @@
 #include "../ut.h"
 #include "../error.h"
 #include "../dprint.h"
+#include "../data_lump.h"
 #include "../data_lump_rpl.h"
 #include "../mem/mem.h"
 #include "../error.h"
@@ -51,6 +52,7 @@
 #include "../core_stats.h"
 #include "../errinfo.h"
 #include "../dset.h"
+#include "../sdp_ops.h"
 #include "parse_hname2.h"
 #include "parse_uri.h"
 #include "parse_content.h"
@@ -68,7 +70,7 @@ int via_cnt;
 
 /* returns pointer to next header line, and fill hdr_f ;
  * if at end of header returns pointer to the last crlf  (always buf)*/
-char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
+char* get_hdr_field_aux(char* buf, char* end, struct hdr_field* hdr,int sip_well_known_parse)
 {
 
 	char* tmp;
@@ -107,88 +109,172 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 			/* keep number of vias parsed -- we want to report it in
 			   replies for diagnostic purposes */
 			via_cnt++;
-			vb=pkg_malloc(sizeof(struct via_body));
-			if (vb==0){
-				LM_ERR("out of pkg memory\n");
-				goto error;
+			if (sip_well_known_parse) {
+				vb=pkg_malloc(sizeof(struct via_body));
+				if (vb==0){
+					LM_ERR("out of pkg memory\n");
+					goto error;
+				}
+				memset(vb,0,sizeof(struct via_body));
+				hdr->body.s=tmp;
+				tmp=parse_via(tmp, end, vb);
+				if (vb->error==PARSE_ERROR){
+					LM_ERR("bad via\n");
+					free_via_list(vb);
+					set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
+						"error parsing Via");
+					set_err_reply(400, "bad Via header");
+					goto error;
+				}
+				hdr->parsed=vb;
+				vb->hdr.s=hdr->name.s;
+				vb->hdr.len=hdr->name.len;
+				hdr->body.len=tmp-hdr->body.s;
+			} else {
+				/* just skip over it */
+				hdr->body.s=tmp;
+				/* find end of header */
+				/* find lf */
+				do{
+					match=q_memchr(tmp, '\n', end-tmp);
+					if (match){
+						match++;
+					}else {
+						LM_ERR("bad body for <%.*s>(%d)\n",
+							 hdr->name.len, hdr->name.s, hdr->type);
+						tmp=end;
+						goto error_bad_hdr;
+					}
+					tmp=match;
+				}while( match<end &&( (*match==' ')||(*match=='\t') ) );
+				tmp=match;
+				hdr->body.len=match-hdr->body.s;
 			}
-			memset(vb,0,sizeof(struct via_body));
-			hdr->body.s=tmp;
-			tmp=parse_via(tmp, end, vb);
-			if (vb->error==PARSE_ERROR){
-				LM_ERR("bad via\n");
-				free_via_list(vb);
-				set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
-					"error parsing Via");
-				set_err_reply(400, "bad Via header");
-				goto error;
-			}
-			hdr->parsed=vb;
-			vb->hdr.s=hdr->name.s;
-			vb->hdr.len=hdr->name.len;
-			hdr->body.len=tmp-hdr->body.s;
 			break;
 		case HDR_CSEQ_T:
-			cseq_b=pkg_malloc(sizeof(struct cseq_body));
-			if (cseq_b==0){
-				LM_ERR("out of pkg memory\n");
-				goto error;
+			if (sip_well_known_parse) {
+				cseq_b=pkg_malloc(sizeof(struct cseq_body));
+				if (cseq_b==0){
+					LM_ERR("out of pkg memory\n");
+					goto error;
+				}
+				memset(cseq_b, 0, sizeof(struct cseq_body));
+				hdr->body.s=tmp;
+				tmp=parse_cseq(tmp, end, cseq_b);
+				if (cseq_b->error==PARSE_ERROR){
+					LM_ERR("bad cseq\n");
+					pkg_free(cseq_b);
+					set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
+						"error parsing CSeq`");
+					set_err_reply(400, "bad CSeq header");
+					goto error;
+				}
+				hdr->parsed=cseq_b;
+				hdr->body.len=tmp-hdr->body.s;
+				LM_DBG("cseq <%.*s>: <%.*s> <%.*s>\n",
+						hdr->name.len, ZSW(hdr->name.s),
+						cseq_b->number.len, ZSW(cseq_b->number.s),
+						cseq_b->method.len, cseq_b->method.s);
+			} else {
+				/* just skip over it */
+				hdr->body.s=tmp;
+				/* find end of header */
+				/* find lf */
+				do{
+					match=q_memchr(tmp, '\n', end-tmp);
+					if (match){
+						match++;
+					}else {
+						LM_ERR("bad body for <%.*s>(%d)\n",
+							 hdr->name.len, hdr->name.s, hdr->type);
+						tmp=end;
+						goto error_bad_hdr;
+					}
+					tmp=match;
+				}while( match<end &&( (*match==' ')||(*match=='\t') ) );
+				tmp=match;
+				hdr->body.len=match-hdr->body.s;
 			}
-			memset(cseq_b, 0, sizeof(struct cseq_body));
-			hdr->body.s=tmp;
-			tmp=parse_cseq(tmp, end, cseq_b);
-			if (cseq_b->error==PARSE_ERROR){
-				LM_ERR("bad cseq\n");
-				pkg_free(cseq_b);
-				set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
-					"error parsing CSeq`");
-				set_err_reply(400, "bad CSeq header");
-				goto error;
-			}
-			hdr->parsed=cseq_b;
-			hdr->body.len=tmp-hdr->body.s;
-			LM_DBG("cseq <%.*s>: <%.*s> <%.*s>\n",
-					hdr->name.len, ZSW(hdr->name.s),
-					cseq_b->number.len, ZSW(cseq_b->number.s),
-					cseq_b->method.len, cseq_b->method.s);
 			break;
 		case HDR_TO_T:
-			to_b=pkg_malloc(sizeof(struct to_body));
-			if (to_b==0){
-				LM_ERR("out of pkg memory\n");
-				goto error;
+			if (sip_well_known_parse) {
+				to_b=pkg_malloc(sizeof(struct to_body));
+				if (to_b==0){
+					LM_ERR("out of pkg memory\n");
+					goto error;
+				}
+				memset(to_b, 0, sizeof(struct to_body));
+				hdr->body.s=tmp;
+				tmp=parse_to(tmp, end,to_b);
+				if (to_b->error==PARSE_ERROR){
+					LM_ERR("bad to header\n");
+					pkg_free(to_b);
+					set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
+						"error parsing To header");
+					set_err_reply(400, "bad header");
+					goto error;
+				}
+				hdr->parsed=to_b;
+				hdr->body.len=tmp-hdr->body.s;
+				LM_DBG("<%.*s> [%d]; uri=[%.*s] \n",
+					hdr->name.len, ZSW(hdr->name.s),
+					hdr->body.len, to_b->uri.len,ZSW(to_b->uri.s));
+				LM_DBG("to body [%.*s]\n",to_b->body.len, ZSW(to_b->body.s));
+			} else {
+				/* just skip over it */
+				hdr->body.s=tmp;
+				/* find end of header */
+				/* find lf */
+				do{
+					match=q_memchr(tmp, '\n', end-tmp);
+					if (match){
+						match++;
+					}else {
+						LM_ERR("bad body for <%.*s>(%d)\n",
+							 hdr->name.len, hdr->name.s, hdr->type);
+						tmp=end;
+						goto error_bad_hdr;
+					}
+					tmp=match;
+				}while( match<end &&( (*match==' ')||(*match=='\t') ) );
+				tmp=match;
+				hdr->body.len=match-hdr->body.s;
 			}
-			memset(to_b, 0, sizeof(struct to_body));
-			hdr->body.s=tmp;
-			tmp=parse_to(tmp, end,to_b);
-			if (to_b->error==PARSE_ERROR){
-				LM_ERR("bad to header\n");
-				pkg_free(to_b);
-				set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
-					"error parsing To header");
-				set_err_reply(400, "bad header");
-				goto error;
-			}
-			hdr->parsed=to_b;
-			hdr->body.len=tmp-hdr->body.s;
-			LM_DBG("<%.*s> [%d]; uri=[%.*s] \n",
-				hdr->name.len, ZSW(hdr->name.s),
-				hdr->body.len, to_b->uri.len,ZSW(to_b->uri.s));
-			LM_DBG("to body [%.*s]\n",to_b->body.len, ZSW(to_b->body.s));
 			break;
 		case HDR_CONTENTLENGTH_T:
-			hdr->body.s=tmp;
-			tmp=parse_content_length(tmp,end, &integer);
-			if (tmp==0){
-				LM_ERR("bad content_length header\n");
-				set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
-					"error parsing Content-Length");
-				set_err_reply(400, "bad Content-Length header");
-				goto error;
+			if (sip_well_known_parse) {
+				hdr->body.s=tmp;
+				tmp=parse_content_length(tmp,end, &integer);
+				if (tmp==0){
+					LM_ERR("bad content_length header\n");
+					set_err_info(OSER_EC_PARSER, OSER_EL_MEDIUM,
+						"error parsing Content-Length");
+					set_err_reply(400, "bad Content-Length header");
+					goto error;
+				}
+				hdr->parsed=(void*)(long)integer;
+				hdr->body.len=tmp-hdr->body.s;
+				LM_DBG("content_length=%d\n", (int)(long)hdr->parsed);
+			} else {
+				/* just skip over it */
+				hdr->body.s=tmp;
+				/* find end of header */
+				/* find lf */
+				do{
+					match=q_memchr(tmp, '\n', end-tmp);
+					if (match){
+						match++;
+					}else {
+						LM_ERR("bad body for <%.*s>(%d)\n",
+							 hdr->name.len, hdr->name.s, hdr->type);
+						tmp=end;
+						goto error_bad_hdr;
+					}
+					tmp=match;
+				}while( match<end &&( (*match==' ')||(*match=='\t') ) );
+				tmp=match;
+				hdr->body.len=match-hdr->body.s;
 			}
-			hdr->parsed=(void*)(long)integer;
-			hdr->body.len=tmp-hdr->body.s;
-			LM_DBG("content_length=%d\n", (int)(long)hdr->parsed);
 			break;
 		case HDR_SUPPORTED_T:
 		case HDR_CONTENTTYPE_T:
@@ -237,6 +323,9 @@ char* get_hdr_field(char* buf, char* end, struct hdr_field* hdr)
 		case HDR_SUCCESS_REPORT_T:
 		case HDR_STATUS_T:
 		case HDR_USE_PATH_T:
+		case HDR_SECURITY_CLIENT_T:
+		case HDR_SECURITY_SERVER_T:
+		case HDR_SECURITY_VERIFY_T:
 		case HDR_OTHER_T:
 			/* just skip over it */
 			hdr->body.s=tmp;
@@ -278,8 +367,6 @@ error:
 	return tmp;
 }
 
-
-
 /* parse the headers and adds them to msg->headers and msg->to, from etc.
  * It stops when all the headers requested in flags were parsed, on error
  * (bad header) or end of headers */
@@ -292,23 +379,15 @@ error:
    give you the first occurrence of a header you are interested in,
    look at check_transaction_quadruple
 */
-int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
+int parse_headers_aux(struct sip_msg* msg, hdr_flags_t flags, int next, int sip_well_known_parse)
 {
 	struct hdr_field *hf;
-	struct hdr_field *itr;
 	char* tmp;
 	char* rest;
 	char* end;
 	hdr_flags_t orig_flag;
 
-#define link_sibling_hdr(_hook, _hdr) \
-	do{ \
-		if (msg->_hook==0) msg->_hook=_hdr;\
-			else {\
-				for(itr=msg->_hook;itr->sibling;itr=itr->sibling);\
-				itr->sibling = _hdr;\
-			}\
-	}while(0)
+#define link_sibling_hdr(_hook, _hdr) _add_last(_hdr, msg->_hook, sibling)
 
 	end=msg->buf+msg->len;
 	tmp=msg->unparsed;
@@ -329,7 +408,7 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 		}
 		memset(hf,0, sizeof(struct hdr_field));
 		hf->type=HDR_ERROR_T;
-		rest=get_hdr_field(tmp, msg->buf+msg->len, hf);
+		rest=get_hdr_field_aux(tmp, msg->buf+msg->len, hf,sip_well_known_parse);
 		switch (hf->type){
 			case HDR_ERROR_T:
 				LM_INFO("bad header field\n");
@@ -511,7 +590,7 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 				link_sibling_hdr(h_via1,hf);
 				msg->parsed_flag|=HDR_VIA_F;
 				LM_DBG("via found, flags=%llx\n", (unsigned long long)flags);
-				if (msg->via1==0) {
+				if (sip_well_known_parse && msg->via1==0) {
 					LM_DBG("this is the first via\n");
 					msg->h_via1=hf;
 					msg->via1=hf->parsed;
@@ -519,11 +598,6 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 						msg->via2=msg->via1->next;
 						msg->parsed_flag|=HDR_VIA2_F;
 					}
-				}else if (msg->via2==0){
-					msg->h_via2=hf;
-					msg->via2=hf->parsed;
-					msg->parsed_flag|=HDR_VIA2_F;
-					LM_DBG("parse_headers: this is the second via\n");
 				}
 				break;
 			case HDR_FEATURE_CAPS_T:
@@ -534,6 +608,18 @@ int parse_headers(struct sip_msg* msg, hdr_flags_t flags, int next)
 				link_sibling_hdr(replaces, hf);
 				msg->parsed_flag |= HDR_REPLACES_F;
 				break;
+			case HDR_SECURITY_CLIENT_T:
+				link_sibling_hdr(security_client, hf);
+				msg->parsed_flag |= HDR_SECURITY_CLIENT_F;
+				break;
+			case HDR_SECURITY_SERVER_T:
+				link_sibling_hdr(security_server, hf);
+				msg->parsed_flag |= HDR_SECURITY_SERVER_F;
+				break;
+			case HDR_SECURITY_VERIFY_T:
+				link_sibling_hdr(security_verify, hf);
+				msg->parsed_flag |= HDR_SECURITY_VERIFY_F;
+				break;
 			default:
 				LM_CRIT("unknown header type %d\n",	hf->type);
 				goto error;
@@ -572,15 +658,10 @@ int clone_headers(struct sip_msg *from_msg, struct sip_msg *to_msg)
 	int hdrs_no, i;
 	struct hdr_field *hdrs;
 	struct hdr_field *hdr;
-	struct hdr_field *itr;
 
 #define link_sibling_hdr_case(_hook, _hdr_type) \
 	case _hdr_type: \
-		if (to_msg->_hook==0) to_msg->_hook=&hdrs[i];\
-		else {\
-			for(itr=to_msg->_hook;itr->sibling;itr=itr->sibling);\
-			itr->sibling = &hdrs[i];\
-		}\
+		_add_last(&hdrs[i], to_msg->_hook, sibling);\
 		break
 #define link_hdr_case(_hook, _hdr_type) \
 	case _hdr_type: \
@@ -603,46 +684,51 @@ int clone_headers(struct sip_msg *from_msg, struct sip_msg *to_msg)
 	}
 
 	/* reset all header fields before populating new ones */
+	to_msg->h_via1 = NULL;
 	to_msg->callid = NULL;
 	to_msg->to = NULL;
 	to_msg->cseq = NULL;
 	to_msg->from = NULL;
-	to_msg->maxforwards = NULL;
-	to_msg->content_type = NULL;
-	to_msg->content_length = NULL;
-	to_msg->expires = NULL;
-	to_msg->organization = NULL;
-	to_msg->priority = NULL;
-	to_msg->subject = NULL;
-	to_msg->user_agent = NULL;
-	to_msg->content_disposition = NULL;
-	to_msg->rpid = NULL;
-	to_msg->refer_to = NULL;
-	to_msg->session_expires = NULL;
-	to_msg->min_se = NULL;
-	to_msg->min_expires = NULL;
-	to_msg->privacy = NULL;
 	to_msg->contact = NULL;
+	to_msg->maxforwards = NULL;
 	to_msg->route = NULL;
 	to_msg->record_route = NULL;
 	to_msg->path = NULL;
+	to_msg->content_type = NULL;
+	to_msg->content_length = NULL;
 	to_msg->authorization = NULL;
+	to_msg->expires = NULL;
 	to_msg->proxy_auth = NULL;
-	to_msg->proxy_require = NULL;
 	to_msg->supported = NULL;
+	to_msg->proxy_require = NULL;
 	to_msg->unsupported = NULL;
 	to_msg->allow = NULL;
 	to_msg->event = NULL;
 	to_msg->accept = NULL;
 	to_msg->accept_language = NULL;
+	to_msg->organization = NULL;
+	to_msg->priority = NULL;
+	to_msg->subject = NULL;
+	to_msg->user_agent = NULL;
+	to_msg->content_disposition = NULL;
 	to_msg->accept_disposition = NULL;
 	to_msg->diversion = NULL;
+	to_msg->rpid = NULL;
+	to_msg->refer_to = NULL;
+	to_msg->session_expires = NULL;
+	to_msg->min_se = NULL;
+	to_msg->ppi = NULL;
+	to_msg->pai = NULL;
+	to_msg->privacy = NULL;
 	to_msg->call_info = NULL;
 	to_msg->www_authenticate = NULL;
 	to_msg->proxy_authenticate = NULL;
-	to_msg->ppi = NULL;
-	to_msg->pai = NULL;
-	to_msg->h_via1 = NULL;
+	to_msg->min_expires = NULL;
+	to_msg->feature_caps = NULL;
+	to_msg->replaces = NULL;
+	to_msg->security_client = NULL;
+	to_msg->security_server = NULL;
+	to_msg->security_verify = NULL;
 
 	for (i = 0, hdr = from_msg->headers; hdr; i++, hdr = hdr->next) {
 		memcpy(&hdrs[i], hdr, sizeof(struct hdr_field));
@@ -650,48 +736,53 @@ int clone_headers(struct sip_msg *from_msg, struct sip_msg *to_msg)
 		hdrs[i].next = &hdrs[i + 1];
 		hdrs[i].sibling = NULL;
 		switch(hdr->type) {
-			link_hdr_case(callid, HDR_CALLID_T);
-			link_hdr_case(to, HDR_TO_T);
-			link_hdr_case(cseq, HDR_CSEQ_T);
-			link_hdr_case(from, HDR_FROM_T);
-			link_hdr_case(maxforwards, HDR_MAXFORWARDS_T);
-			link_hdr_case(content_type, HDR_CONTENTTYPE_T);
-			link_hdr_case(content_length, HDR_CONTENTLENGTH_T);
-			link_hdr_case(expires, HDR_EXPIRES_T);
-			link_hdr_case(organization, HDR_ORGANIZATION_T);
-			link_hdr_case(priority, HDR_PRIORITY_T);
-			link_hdr_case(subject, HDR_SUBJECT_T);
-			link_hdr_case(user_agent, HDR_USERAGENT_T);
-			link_hdr_case(content_disposition, HDR_CONTENTDISPOSITION_T);
-			link_hdr_case(rpid, HDR_RPID_T);
-			link_hdr_case(refer_to, HDR_REFER_TO_T);
-			link_hdr_case(session_expires, HDR_SESSION_EXPIRES_T);
-			link_hdr_case(min_se, HDR_MIN_SE_T);
-			link_hdr_case(min_expires, HDR_MIN_EXPIRES_T);
-			link_hdr_case(privacy, HDR_PRIVACY_T);
+			link_sibling_hdr_case(h_via1, HDR_VIA_T);
+			link_hdr_case        (callid, HDR_CALLID_T);
+			link_hdr_case        (to, HDR_TO_T);
+			link_hdr_case        (cseq, HDR_CSEQ_T);
+			link_hdr_case        (from, HDR_FROM_T);
 			link_sibling_hdr_case(contact, HDR_CONTACT_T);
+			link_hdr_case        (maxforwards, HDR_MAXFORWARDS_T);
 			link_sibling_hdr_case(route, HDR_ROUTE_T);
 			link_sibling_hdr_case(record_route, HDR_RECORDROUTE_T);
 			link_sibling_hdr_case(path, HDR_PATH_T);
+			link_hdr_case        (content_type, HDR_CONTENTTYPE_T);
+			link_hdr_case        (content_length, HDR_CONTENTLENGTH_T);
 			link_sibling_hdr_case(authorization, HDR_AUTHORIZATION_T);
+			link_hdr_case        (expires, HDR_EXPIRES_T);
 			link_sibling_hdr_case(proxy_auth, HDR_PROXYAUTH_T);
-			link_sibling_hdr_case(proxy_require, HDR_PROXYREQUIRE_T);
 			link_sibling_hdr_case(supported, HDR_SUPPORTED_T);
+			link_sibling_hdr_case(proxy_require, HDR_PROXYREQUIRE_T);
 			link_sibling_hdr_case(unsupported, HDR_UNSUPPORTED_T);
 			link_sibling_hdr_case(allow, HDR_ALLOW_T);
 			link_sibling_hdr_case(event, HDR_EVENT_T);
 			link_sibling_hdr_case(accept, HDR_ACCEPT_T);
 			link_sibling_hdr_case(accept_language, HDR_ACCEPTLANGUAGE_T);
+			link_hdr_case        (organization, HDR_ORGANIZATION_T);
+			link_hdr_case        (priority, HDR_PRIORITY_T);
+			link_hdr_case        (subject, HDR_SUBJECT_T);
+			link_hdr_case        (user_agent, HDR_USERAGENT_T);
+			link_hdr_case        (content_disposition, HDR_CONTENTDISPOSITION_T);
 			link_sibling_hdr_case(accept_disposition, HDR_ACCEPTDISPOSITION_T);
 			link_sibling_hdr_case(diversion, HDR_DIVERSION_T);
+			link_hdr_case        (rpid, HDR_RPID_T);
+			link_hdr_case        (refer_to, HDR_REFER_TO_T);
+			link_hdr_case        (session_expires, HDR_SESSION_EXPIRES_T);
+			link_hdr_case        (min_se, HDR_MIN_SE_T);
+			link_sibling_hdr_case(ppi, HDR_PPI_T);
+			link_sibling_hdr_case(pai, HDR_PAI_T);
+			link_hdr_case        (privacy, HDR_PRIVACY_T);
 			link_sibling_hdr_case(call_info, HDR_CALL_INFO_T);
 			link_sibling_hdr_case(www_authenticate, HDR_WWW_AUTHENTICATE_T);
 			link_sibling_hdr_case(proxy_authenticate, HDR_PROXY_AUTHENTICATE_T);
-			link_sibling_hdr_case(ppi, HDR_PPI_T);
-			link_sibling_hdr_case(pai, HDR_PAI_T);
-			link_sibling_hdr_case(h_via1, HDR_VIA_T);
+			link_hdr_case        (min_expires, HDR_MIN_EXPIRES_T);
+			link_sibling_hdr_case(feature_caps, HDR_FEATURE_CAPS_T);
+			link_hdr_case        (replaces, HDR_REPLACES_T);
+			link_sibling_hdr_case(security_client, HDR_SECURITY_CLIENT_T);
+			link_sibling_hdr_case(security_server, HDR_SECURITY_SERVER_T);
+			link_sibling_hdr_case(security_verify, HDR_SECURITY_VERIFY_T);
 
-			/* not used in sip_msg */
+			/* not used in sip_msg, used in MSRP */
 			case HDR_TO_PATH_T:
 			case HDR_FROM_PATH_T:
 			case HDR_MESSAGE_ID_T:
@@ -701,12 +792,22 @@ int clone_headers(struct sip_msg *from_msg, struct sip_msg *to_msg)
 			case HDR_STATUS_T:
 			case HDR_USE_PATH_T:
 
+			/* not having shortcut */
+			case HDR_RETRY_AFTER_T:
+			case HDR_VIA2_T:
+
+			/* not actual hdrs */
 			case HDR_OTHER_T:
 			case HDR_ERROR_T:
+			case HDR_EOH_T:
 				break;
+			/* we do not have a "default" on purpose, so we get
+			 * a compile err/war where a new HDR is added and we do 
+			 * not handle it here.
 			default:
 				LM_ERR("unknown header type %d\n", hdr->type);
 				break;
+			*/
 		}
 	}
 	hdrs[i - 1].next = 0;
@@ -718,7 +819,8 @@ int clone_headers(struct sip_msg *from_msg, struct sip_msg *to_msg)
 
 
 /* returns 0 if ok, -1 for errors */
-int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
+int parse_msg_opt(char* buf, unsigned int len, struct sip_msg* msg,
+															int free_on_err)
 {
 
 	char *tmp;
@@ -772,7 +874,7 @@ int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
 	}
 	msg->unparsed=tmp;
 	/*find first Via: */
-	if (parse_headers(msg, flags, 0)==-1) goto error;
+	if (parse_headers(msg, flags, 0)==-1) goto parse_error;
 
 #ifdef EXTRA_DEBUG
 	/* dump parsed data */
@@ -824,6 +926,14 @@ int parse_msg(char* buf, unsigned int len, struct sip_msg* msg)
 
 	return 0;
 
+parse_error:
+	if (free_on_err) {
+		/* Free the SIP message if it has already been created to
+		 * prevent memory leaks */
+		free_sip_msg(msg);
+		memset(msg, 0, sizeof(*msg));
+	}
+
 error:
 	/* more debugging, msg->orig is/should be null terminated*/
 	LM_ERR("message=<%.*s>\n", (int)len, ZSW(buf));
@@ -866,6 +976,8 @@ void free_sip_msg(struct sip_msg* msg)
 		free_lump_list(msg->add_rm);
 	if (msg->body_lumps)
 		free_lump_list(msg->body_lumps);
+	if (msg->sdp_ops )
+		free_sdp_ops(msg->sdp_ops);
 	if (msg->reply_lump)
 		free_reply_lump(msg->reply_lump);
 	if (msg->body )
@@ -1424,3 +1536,15 @@ done:
 }
 
 
+int delete_headers(struct sip_msg *msg, struct hdr_field *hdr)
+{
+	for (; hdr; hdr = hdr->sibling) {
+		if (!del_lump(msg, hdr->name.s - msg->buf, hdr->len, hdr->type)) {
+			LM_ERR("failed to delete contact '%.*s'\n", hdr->name.len,
+			       hdr->name.s);
+			return -1;
+		}
+	}
+
+	return 0;
+}
